
/***************************************************************************
                           irc.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _IRC_HPP
#define _IRC_HPP

#include <iostream>
#include <string>
#include <config.h>

using namespace std;

/*
  todo:
  ^^^^^
  implement copy constructor
  implement copy operator
*/
// typedef enum {
//   UNSET=-1, HOSTMASK, CMD, TARGET, OPT1, OPT2, OPT3, OPT4, OPT5, OPT6, OPT7, OPT8, OPT9, OPT10, OPT11, OPT12, OPT13, MSG 
// } ircoptidx_t;

typedef short ircoptidx_t;
const static ircoptidx_t IDX_UNSET=-1;
const static ircoptidx_t IDX_HOSTMASK=0;
const static ircoptidx_t IDX_CMD=1;
const static ircoptidx_t IDX_TARGET=2;
const static ircoptidx_t IDX_OPT1=3;
const static ircoptidx_t IDX_OPT2=4;
const static ircoptidx_t IDX_OPT3=5;
const static ircoptidx_t IDX_OPT4=6;
const static ircoptidx_t IDX_OPT5=7;
const static ircoptidx_t IDX_OPT6=8;
const static ircoptidx_t IDX_OPT7=9;
const static ircoptidx_t IDX_OPT8=10;
const static ircoptidx_t IDX_OPT9=11;
const static ircoptidx_t IDX_OPT10=12;
const static ircoptidx_t IDX_OPT11=13;
const static ircoptidx_t IDX_OPT12=14;
const static ircoptidx_t IDX_OPT13=15;
const static ircoptidx_t IDX_MSG=16;

class IrcToken : public string {
private:
  ircoptidx_t type;
  
public:
  IrcToken(ircoptidx_t t=IDX_UNSET, const char *str="") : string(str), type(t) {};
  IrcToken(ircoptidx_t t, const char *str, int len) : string(str, len), type(t) {};
  IrcToken(ircoptidx_t t, const string & str) : string(str), type(t) {};
  const IrcToken& operator= (const IrcToken &tok);
  bool operator ==(ircoptidx_t t) const { return type==t; };
  // operator ircoptidx_t() const { return type; };
};

class IrcMessage {
private:
  const static int OPTIONS_LEN=17;

  string raw;
  bool serverMessage;

  string senderNick;
  string senderIdent;
  string senderAddress;
  ircoptidx_t maxLen; // number of actually set options
  IrcToken *options;

public:
  IrcMessage();
  IrcMessage(const string &unparsed);
  IrcMessage(const IrcMessage &msg);
  const IrcMessage & operator=(const string &);
  const IrcMessage & operator=(const IrcMessage &);
  const IrcMessage & operator=(const char *raw);
  ~IrcMessage() { delete [] options; };

  void assign(const string &raw);
  void assign(const char *raw);

  const string & getRaw() const { return raw; };
  const IrcToken & getToken(ircoptidx_t idx) const;
  const string & getSenderNick() const { return senderNick; };
  const string & getSenderIdent() const { return senderIdent; };
  const string & getSenderAddress() const { return senderAddress; };
  const IrcToken & getHostmask() const { return getToken(IDX_HOSTMASK); };
  const IrcToken & getCommand() const { return getToken(IDX_CMD); };
  const IrcToken & getMessage() const { return getToken(maxLen); };

  bool isSpecial() const { return serverMessage; };

  ircoptidx_t end() const { return maxLen+1; };
  ircoptidx_t begin() const { if(serverMessage) return IDX_CMD; else return IDX_HOSTMASK; };
  ircoptidx_t length() const { return maxLen+1; };

private:
  ircoptidx_t parse(const char * raw);
  bool parseHostmask(const string & hostmask); // return: servermessage
  void copyArray(IrcToken *start);
};

ostream & operator << (ostream &, const IrcMessage &);

#endif
