
/***************************************************************************
                           irconnection.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _IRCCONNECTION_HPP
#define _IRCCONNECTION_HPP

#include <tcpconnection.hpp>
#include <iostream>
#include <string>
#include <config.h>

using namespace std;

class IRCConnection : public TCPConnection {
public:
  IRCConnection(const PSInetHostAddress &addr, tpport_t port, bool autorun=false); 
  ~IRCConnection();

  void printWallop(const string & text);
  void printPong(const string & text);
  void printPing(const string & text);
  void printMsg(const string &target, const string & text);
  void printNotice(const string & target, const string & text);
  void printCommand(const string & command, const string & text);
  void printCommand(const string & command, const string & target, const string &text);
  
  const static string CMD_WALLOP;
  const static string CMD_MSG;
  const static string CMD_NOTICE;
  const static string CMD_PING;
  const static string CMD_PONG;
  
protected:  
  virtual void onConnect() 
  {};
  virtual void onPolling();
  virtual void onInput(char *buf, int len)=0;
  virtual void onDisconnected
  ();
  virtual void onOverflow()
  {}

private:
  char *buf;
  static const int BUF_SIZE;
  
  void flushInput();
  IRCConnection();
  IRCConnection(const IRCConnection &);
  const IRCConnection & operator=(const IRCConnection&);
};

#endif
