
/***************************************************************************
                           ircserverconnection.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _IRCSERVERCONNECTION_HPP
#define _IRCSERVERCONNECTION_HPP

class IRCServerConnection;
#include <ircconnection.hpp>
#include <irc.hpp>
#include <config.h>

using namespace std;

class IRCServerConnection : public IRCConnection {
public:
  IRCServerConnection( const PSInetHostAddress &addr, tpport_t port, 
		       const string & servname, const string &Password );
  virtual ~IRCServerConnection();

  void introduceUser( const string &nick, const string &ident, 
		      const string &address, const string &modes, 
		      const string &realname );  
  void removeUser( const string &nick );
  void joinChannel( const string &nick, const string &chan );
  void quitFromServer( const char *cause );
  static void setDefaultServerDescription( const string &newDescr );

protected:
  virtual void onConnected() {} // will be called by onConnect()
  virtual void onPrivmsg(const IrcMessage &) {};
  virtual void onNotice(const IrcMessage &) {};
  virtual void onCtcp(const IrcMessage &) {}; // not implemented yet
  virtual void onMode(const IrcMessage &) {};
  virtual void onError(const IrcMessage &) {};
  virtual void onNickChange(const IrcToken &oldNick, const IrcToken &newNick, const IrcToken &timestamp) {}
  virtual void onNickIntroduce(const IrcToken &nick, const IrcToken &ident, const IrcToken &address, 
			       const IrcToken &realname, const IrcToken &modes, const IrcToken &timestamp) {}
  virtual void onQuit(const IrcMessage &) {};
  void printMsg(const string &sender, const string &target, const string &msg);
  void printNotice(const string &sender, const string &target, const string &msg);

private:
  virtual void onConnect();
  virtual void onDisconnected();
  virtual void onInput(char *buf, int len);

  const string password;
  const string hostname;
  IrcMessage ircMsg;
  timeout_t timeout;
  void process(const IrcMessage &msg);

  static long DEF_TIMEOUT;
  static string SERVER_DESCR;

  IRCServerConnection();
  IRCServerConnection(const IRCServerConnection &);
  const IRCServerConnection & operator=(const IRCServerConnection &);
};



#endif
