/***************************************************************************
                           processalias.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _PROCESSALIAS_HPP
#define _PROCESSALIAS_HPP

#include <string>
#include <iostream>
#include <config.h>

using namespace std;

class ProcessAlias {
public:
  ProcessAlias();
  ProcessAlias( const string &command, const string &options,
		const string &ircnick, 
		const string &ircIdent=DEFAULT_IDENT, 
		const string &ircAddress=DEFAULT_ADDRESS,
		const string &ircRealname=DEFAULT_REALNAME, 
		const string &ircModes=DEFAULT_MODES, 
		const string &ircChannel=DEFAULT_CHANNEL,
		bool overwriteArgs=OVERWRITE_OPTIONS );
  virtual ~ProcessAlias();

  const string &getCommand() const 
  { return command; };
  const string &getOptions() const 
  { return options; };
  const string &getIRCName() const 
  { return ircNick; };
  const string &getIRCIdent() const
  { return ircIdent; };
  const string &getIRCAddress() const
  { return ircAddress; };
  const string &getIRCRealname() const
  { return ircRealname; };
  const string &getIRCModes() const
  { return ircModes; };
  const string &getIRCChannel() const 
  { return ircChannel; };
  bool isEmpty() const 
  { return command.empty(); };
  bool overwriteOptions() const
  { return overwrite; };
  int getReferences() const
  { return references; };

  // !!! warning. these functions must be used in pair to avoid 
  // memory leaks !!!
  void incrementReferences() const
  { ++references ; };
  void decrementReferences() const
  { --references ; };

  static void setDefaultChannel(const string &chan) 
  { DEFAULT_CHANNEL = chan; };
  static void setDefaultIdent(const string &ident)
  { DEFAULT_IDENT = ident; };
  static void setDefaultAddress(const string &address)
  { DEFAULT_ADDRESS = address; };
  static void setDefaultRealname(const string &realname)
  { DEFAULT_REALNAME = realname; };
  static void setDefaultModes( const string &modes )
  { DEFAULT_MODES = modes; };
  static void setOverwriteOptions( bool overwrite)
  { OVERWRITE_OPTIONS = overwrite; };
  static const string &getDefaultChannel()
  { return DEFAULT_CHANNEL; };
  static const string &getDefaultIdent()
  { return DEFAULT_IDENT; };
  static const string &getDefaultAddress()
  { return DEFAULT_ADDRESS; };
  static const string &getDefaultRealname()
  { return DEFAULT_REALNAME; };
  static const string &getDefaultModes()
  { return DEFAULT_MODES; };
  static bool getDefaultOverwriteOptions()
  { return OVERWRITE_OPTIONS; };
  
private:
  const string command;
  string options;
  const string ircNick;
  const string ircIdent;
  const string ircAddress;
  const string ircRealname;
  const string ircModes;
  const string ircChannel;
  const bool overwrite;
  mutable int references;

  static string DEFAULT_IDENT;
  static string DEFAULT_ADDRESS;
  static string DEFAULT_MODES;
  static string DEFAULT_REALNAME;
  static string DEFAULT_CHANNEL;
  static bool OVERWRITE_OPTIONS;

  const ProcessAlias & operator= ( const ProcessAlias &p );
  ProcessAlias( const ProcessAlias & );
};

ostream & operator << (ostream &out, const ProcessAlias &pa);

#endif
