
/***************************************************************************
                           psircconnection.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _PSIRCCONNECTION_HPP
#define _PSIRCCONNECTION_HPP

class PSIRCConnection;
#include <string>
#include <ircserverconnection.hpp>
#include <clientlist.hpp>
#include <clienttaskset.hpp>
#include <pstask.hpp>
#include <logins.hpp>
#include <vprogs.hpp>
#include <config.h>

//find a better solution for defaultUserName
class PSIRCConnection : public IRCServerConnection 
{
public:
  PSIRCConnection (const procserv::PSInetHostAddress &addr, tpport_t port,
		  const string &servname, const string &password,
		  VProgs *vprogs, Logins *authData);
  virtual ~PSIRCConnection();

  // to enable rehash functionality
  void setNewAuthData( Logins *l );
  void setNewVProgs( VProgs *v );

  static void setDefaultManagerNick( const string &str );
  static void setDefaultManagerIdent( const string &str );
  static void setDefaultManagerAddress( const string &str );
  static void setDefaultManagerModes( const string &str );
  static void setDefaultManagerRealname( const string &str );
  static void setDefaultManagerUserMode( const string &str );
  
protected:
  virtual void onPrivmsg( const IrcMessage & );
  virtual void onNotice( const IrcMessage & );
  virtual void onConnected();
  virtual void onError( const IrcMessage & );
  virtual void onQuit( const IrcMessage & );
  virtual void onNickChange( const IrcToken &oldNick, 
			     const IrcToken &newNick, 
			     const IrcToken &timestamp );

private:
  ClientList *ircClients; // list of authed clients
  ClientTaskSet *tasks; // list of running tasks
  VProgs *vprogs; // list of available vprogs
  Logins *authData; // list of logins
  string defaultUserName; // manager ircname
 
  const static string CMD_TASKADD;
  const static string CMD_TASKHUP;
  const static string CMD_TASKDEL;
  const static string CMD_TASKKILL;
  const static string CMD_TASKCOMM;
  const static string CMD_SYSINFO;
  const static string CMD_AUTH;
  const static string CMD_DEAUTH;
  const static string CMD_REHASH;
  const static string CMD_LISTVPROGS;
  const static string CMD_HELP;

  static string MANAGER_NICK;
  static string MANAGER_IDENT;
  static string MANAGER_ADDRESS;
  static string MANAGER_MODES;
  static string MANAGER_REALNAME;

  void processManagerMsg( const IrcMessage & );
  void processProcessMsg( const IrcMessage & );
  void processTaskAdd( const string &sender, const string &type, 
		       const string &args, bool argsSet );
  void processTaskDel( PSTask *t );
  void processTaskKill( PSTask *t );
  void processTaskHup( PSTask *t );
  void processTaskComm( PSTask *t, const string &msg );
  void processAuth( const string &sender, const string &username, const string &password );
  void processRehash( const string &sender );
  void processListVProgs( const string &sender );
  bool containsIgnoreCase( string::const_iterator hbegin, 
			   string::const_iterator hend,
			   string::const_iterator nbegin,
			   string::const_iterator nend );
  int fillTokenArray( const string &, string *, int len, char delim=' ' );
  string parseProcessArgs( const string &origOptions, const string &optOpts );

  void printNoSuchCommand( const string &target ); // maybe improve a bit
  void printLoginFirst( const string &target );
  void printTaskHelp( const string &target );
  void printManagerHelp( const string &target );
  void printSysInfo( const string &target );
  void printDeAuthFailed( const string & target );
  void printDeAuthDone( const string &target );
  void printAuthDone( const string &target );
  void printAuthFailed( const string &target );
  void printLoginHelp( const string &target );
  void printAlreadyLogedin( const string &target );
  void printRehashed( const string &target );
  void printVProg( const string &vprogName, const ProcessAlias &alias, const string &target );
  void printNoSuchVProg( const string &target, const string &vprogName );

  PSIRCConnection();
  PSIRCConnection( const PSIRCConnection & );
  const PSIRCConnection & operator=( const PSIRCConnection& );
};

#endif
