
/***************************************************************************
                           tcpconnection.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _TCPCONNECTION_HPP
#define _TCPCONNECTION_HPP

#include <iostream>
#include <inethostaddress.hpp>
#include <spstream.hpp>
extern "C" {
#include <sys/poll.h>
}
#include <config.h>

using namespace std;
using namespace procserv;

typedef unsigned short tpport_t;
typedef unsigned long timeout_t;


// 'caching' class - once you have specified the address you do not need to 
//                   specify it once more, unless the address should be 
//                   changed
class TCPConnection {
public:
  TCPConnection();
  TCPConnection(const PSInetHostAddress &addr, tpport_t port);
  virtual ~TCPConnection();
  
  int Recv(char *buf, size_t len); // blocking
  int Peek(char *buf, size_t len);
  int Write(const char *buf, size_t len);
  void startPolling();
  void Close();
  void Connect(const PSInetHostAddress &addr, tpport_t port);
  void Connect();

protected:
  void stopPolling() { runPoll=false; };
  virtual void onConnect() 
  { cout << "TCPConnection] onConnect()" << endl; };
  virtual void onDisconnected()
  { cout << "TCPConnection] onDisconnected" << endl; };
  virtual void onTimeout() 
  { cout << "TCPConnection] onTimeout()" << endl; };
  virtual void onPolling() 
  { cout << "TCPConnection] onPolling()" << endl; };
  spofstream *tcp() { return tcpStream; };

private:
  int sd;
  bool runPoll;
  spofstream* tcpStream;
  PSInetHostAddress addr;
  tpport_t port;

  void initPollFD(pollfd *f[]);
  const TCPConnection & operator=(const TCPConnection &);
  TCPConnection(const TCPConnection &);
};

#endif
