
/***************************************************************************
                           xmlentity.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#ifndef _XMLENTITY_HPP
#define _XMLENTITY_HPP

class XMLEntity;
#include <xmltoken.hpp>
#include <map>
#include <list>
#include <iostream>
#include <config.h>

using namespace std;

typedef map<const XMLToken, XMLToken> XMLEntityMap;

class XMLEntity : public XMLEntityMap {
public:
  typedef list<XMLEntity *> SubEntitiesList;  

  XMLEntity(XMLEntity * const Root, const XMLToken & Type)
    : XMLEntityMap(), root(Root), type(Type)
  { 
    if(Root) Root->l.push_back(const_cast<XMLEntity*>(this)); 
#ifdef VERBOSE_MEMORY
    cout << "XMLEntity] created for " << Type << endl;
#endif
  }

  XMLEntity(XMLEntity * const Root, const char *Type, int len) 
    : XMLEntityMap(), root(Root), type(Type, len)
  {
    if(Root) Root->l.push_back(const_cast<XMLEntity*>(this));
#ifdef VERBOSE_MEMORY
    cout << "XMLEntity] created for " << Type << endl;
#endif
  }

  virtual ~XMLEntity() 
  {
#ifdef VERBOSE_MEMORY
    cout << "XMLEntity] destroyed" << endl;
#endif
  };

  void addDescription(const XMLToken &t, const XMLToken &k)
  {     
    insert(make_pair(t, k)); 
#ifdef DEBUG
    cout << "XMLEntity] adding description for " << type << ": " << t << "->" << k << endl;
#endif
  }

  bool operator==(const XMLEntity &e) const
  { return type==e.type; };
  
  bool operator<(const XMLEntity &e) const 
  { return type < e.type; };

  bool operator>(const XMLEntity &e) const
  { return type > e.type; };

  const XMLToken& getType() const 
  { return type; };
  
  const SubEntitiesList & getSubEntities() const
  { return l; };

  const XMLEntity * const getRoot() 
  { return root; };

private:
  const XMLEntity * const root;
  const XMLToken type;
  SubEntitiesList l;
};

ostream & operator <<(ostream &out, const XMLEntity &entity);

#endif
