
/***************************************************************************
                           irc.cpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include <irc.hpp>
#include <exception>

const IrcToken& IrcToken::operator=(const IrcToken &tok)
{
  if(this==&tok) return *this;
    this->assign(tok);
    type=tok.type;
    return *this;
}

IrcMessage::IrcMessage() :  
  raw(), serverMessage(false), senderNick(), 
  senderIdent(), senderAddress(), maxLen(IDX_UNSET)  
{
  options=new IrcToken[OPTIONS_LEN];
};

IrcMessage::IrcMessage(const string &unparsed) :
  raw(), serverMessage(false), senderNick(), 
  senderIdent(), senderAddress(), maxLen(IDX_UNSET) 
{
  options=new IrcToken[OPTIONS_LEN];
  assign(unparsed);
};

IrcMessage::IrcMessage(const IrcMessage & msg) : 
  raw(msg.raw), serverMessage(msg.serverMessage), 
  senderNick(msg.senderNick), senderIdent(msg.senderIdent), 
  senderAddress(msg.senderAddress), maxLen(msg.maxLen) 
{
  options=new IrcToken[OPTIONS_LEN];
  copyArray(&(msg.options[0]));
};

void IrcMessage::assign(const char *unparsed)
{
  raw=unparsed;
  maxLen=parse(unparsed);
  serverMessage=parseHostmask(options[IDX_HOSTMASK]);
}

void IrcMessage::assign(const string &unparsed) {
  assign(unparsed.c_str());
};

const IrcMessage & IrcMessage::operator=(const string &unparsed) {
  assign(unparsed);
  return *this;
}

const IrcMessage & IrcMessage::operator=(const char *unparsed) {
  assign(unparsed);
  return *this;
}

const IrcMessage & IrcMessage::operator=(const IrcMessage &msg) {
  if(&msg == this) return *this;

  raw=msg.raw;
  serverMessage=msg.serverMessage;
  senderNick=msg.senderNick;
  senderIdent=msg.senderIdent;
  senderAddress=msg.senderAddress;
  maxLen=msg.maxLen;
  copyArray(&msg.options[0]);
  return *this;
}

const IrcToken & IrcMessage::getToken(ircoptidx_t idx) const {
  // if(idx<0 || idx>=OPTIONS_LEN) throw std::out_of_range();
  OUTOFRANGE( idx < 0 ); // bad code - *selfslap*
  OUTOFRANGE( idx >=OPTIONS_LEN ); // bad code *selfslap*
  return options[idx];
};

// ircoptidx_t IrcMessage::parse(const string &raw)
// {
//   string::size_type begin=(string::size_type)0;
//   ircoptidx_t index=IDX_HOSTMASK;

//   if(raw[0]!=':') {
//     options[index]=IrcToken();
//     ++index;
//     serverMessage=true;
//   } else {
//     serverMessage=false;
//     ++begin;
//   }

//   string::size_type end=begin;
//   for(; (index<(OPTIONS_LEN-1)) && (end=raw.find(' ', begin))!=string::npos; ++index) {
//     if(raw[begin]==':') {
//       ++begin;
//       break;
//     }
//     int rlen=end-begin;
//     if(rlen>0) options[index]=IrcToken(index, raw.substr(begin, rlen)); // fix for 
//     else --index;
//     begin=++end;
//   }
//   options[index]=IrcToken(index, raw.substr(begin, end-begin));
//   return ++index;
// };

ircoptidx_t IrcMessage::parse(const char *raw)
{
  const char *beg=raw;
  ircoptidx_t idx=0;

  if(*beg!=':') {
    // options[index]=IrcToken();
    ++idx;
    serverMessage=true;
  } else {
    serverMessage=false;
    ++beg;
  }

  string tempStr;
  for(;;++idx) {
    int len=0;
    if(*beg==':') {
      ++beg;
      break;
    }
    for(const char *c=beg; beg && *c!=' ';++len, ++c);
    if(!beg) break;

    options[idx]=IrcToken(idx, beg, len);
    beg+=len;

    for(;beg && *beg==' '; ++beg);
    if(!beg) break;
  }
  IrcToken lastToken(idx, beg);
  lastToken.erase(lastToken.length()-1);
  options[idx]=lastToken;
  return idx;
}

bool IrcMessage::parseHostmask(const string & hostmask)
{
  string::size_type idx1=hostmask.find('!');
  if(idx1==string::npos) {
    senderAddress=hostmask;
    senderIdent="";
    senderNick="";
    return true;
  }
  senderNick=hostmask.substr((string::size_type)0, idx1);

  string::size_type idx2=hostmask.find('@', ++idx1);
  senderIdent=hostmask.substr(idx1, idx2-idx1);
  senderAddress=hostmask.substr(++idx2);
  return false;
};

void IrcMessage::copyArray(IrcToken *foreign) 
{
  int i=0;
  if(serverMessage) {
    ++i;
    ++foreign;
  }
  IrcToken *opt=&options[i];
  
  for(;i<maxLen; ++i, ++foreign, ++opt) *opt=*foreign;
}

ostream & operator << (ostream & out, const IrcMessage &msg)
{
  for(ircoptidx_t it=msg.begin(); it<msg.end(); ++it) out << msg.getToken(it) << ' ';
  return out;
}
