
/***************************************************************************
                           ircconnection.cpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cstdlib>
#include <ircconnection.hpp>
#include <iostream>

const string IRCConnection::CMD_WALLOP("WALLOPS");
const string IRCConnection::CMD_MSG("PRIVMSG");
const string IRCConnection::CMD_NOTICE("NOTICE");
const string IRCConnection::CMD_PING("PING");
const string IRCConnection::CMD_PONG("PONG");
const int IRCConnection::BUF_SIZE=1027;

IRCConnection::IRCConnection(const PSInetHostAddress &addr, tpport_t port, bool autorun)
  : TCPConnection(addr, port), buf(new char[BUF_SIZE+1])
{
#ifdef VERBOSE_MEMORY
  cout << "IRCConnection] created" << endl;
#endif
  if(autorun) startPolling();
};

IRCConnection::~IRCConnection()
{
  delete[] buf;
#ifdef VERBOSE_MEMORY
  cout << "IRCConnection] destroyed" << endl;
#endif
}

void IRCConnection::onPolling()
{
  int len=Peek( buf, BUF_SIZE - 2 );
  
  if( len > BUF_SIZE-2 ) {
    onOverflow();
    flushInput();
  }
  else if( len > 0 ) {
    for( int i = 0; i < len; ++i ) {
#if SERVER_TYPE == SERVER_BAHAMUT
      if( buf[i] == '\n' ) {
	++i;
	Recv(buf, i);
	buf[i] = '\0';
	buf[i-1] = '\n';
	onInput(buf, i);
	break;
      }
#elif SERVER_TYPE == SERVER_HYBRID
      if( buf[i] == '\r' && (i+1) < len && buf[i+1]=='\n' ) {
	i+=2;              // this algorithm removes the '\r' an is therefore used
	Recv(buf, i);
	buf[i-1]='\0';
	buf[i-2]='\n';
	onInput(buf, i-1);
	break;
      }
#endif
    }
  }
  else stopPolling();
}

void IRCConnection::onDisconnected()
{
  stopPolling();
}

void IRCConnection::printWallop(const string & text)
{
  printCommand(CMD_WALLOP, text);
};

void IRCConnection::printPong(const string &text)
{
  printCommand(CMD_PONG, text);
}

void IRCConnection::printPing(const string &text)
{
  printCommand(CMD_PING, text);
}

void IRCConnection::printMsg(const string & target, const string &text)
{
  printCommand(CMD_MSG, target, text);
}

void IRCConnection::printNotice(const string & target, const string &text)
{
  printCommand(CMD_NOTICE, target, text);
}

void IRCConnection::printCommand(const string & command, const string &text)
{
  *tcp() << command << " :" << text << endl;
};

void IRCConnection::printCommand(const string &command, const string &target, const string &message)
{
  *tcp() << command << ' ' << target << " :" << message << endl;
}

void IRCConnection::flushInput()
{
  while( Recv(buf, BUF_SIZE) > 0 );
}
