#include <logins.hpp>

const string Logins::XMLENTITY_NAME          = "Auth";
const string Logins::XMLENTITY_USER_NAME     = "User";
const string Logins::XMLENTITY_USER_AUTHNAME = "Name";
const string Logins::XMLENTITY_USER_PASSWORD = "Password";

Logins::Logins() 
  : map<const string, string> ()
{
#ifdef VERBOSE_MEMORY
  cout << "Logins] created" << endl;
#endif
}

Logins::Logins( const XMLEntity *e )
{
#ifdef VERBOSE_MEMORY
  cout << "Logins] created" << endl;
#endif
  process( e );
}

Logins::~Logins()
{
#ifdef LOGINS
  cout << "Logins] destroyed" << endl;
#endif
}

bool Logins::process( const XMLEntity *e )
{
  bool ret = false;
  if( e->getType() == XMLENTITY_NAME ) {
    processSubXMLEntities( e->getSubEntities().begin(),
			   e->getSubEntities().end() );
    ret = true;
  } 

  return ret;
}

void Logins::processSubXMLEntities( XMLEntity::SubEntitiesList::const_iterator begin,
				    XMLEntity::SubEntitiesList::const_iterator end )
{
  for(XMLEntity::SubEntitiesList::const_iterator it = begin; it != end; ++it ) {
    if( (*it)->getType() == XMLENTITY_USER_NAME )
      processUserXMLEntity( *(const_cast<XMLEntity*>( *it )) );
#ifdef DEBUG
    else cout << "Logins] skipping entity: " << (*it)->getType() << endl;
#endif
  }
}

void Logins::processUserXMLEntity( XMLEntity &e )
{
  const string &username = e[XMLENTITY_USER_AUTHNAME];
  const string &password = e[XMLENTITY_USER_PASSWORD];
  if(!e.empty()) {
    insert( make_pair( username, password ) );
#ifdef DEBUG
    cout << "Logins] adding: '" << username << "'->'" << password << '\'' << endl;
#endif
  }
#ifdef DEBUG
  else cout << "Logins] skipping empty entry." << endl;
#endif
}
