
/***************************************************************************
                           main.cpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// TODO: implement cleaner rehash . 
// -> Allocate new datastructures and assign them to procserv

#include <fstream>
extern "C" {
#include <unistd.h>
#include <signal.h>
}

#include <main.hpp>

static const char *OPT_XMLFILE=    "-xmlfile";
static const char *OPT_HELP=       "-h";

static string xmlFile=      XMLFILE;
static VProgs *vprogs=      NULL;
static Logins *logins=      NULL;
static PSConfig *psconfig=  NULL;
static ProcServ *serv=      NULL;

int main(int argc, char **argv)
{
  parseArgs(argc, argv);
  createFromXML();

//   cout << "Doing basic configuration checks ... ";
//   checkConfigFile(*psconfig);
//   cout << "done" << endl;
    
  cout << "Adding signal handlers... ";
  addSignalHandlers();
  cout << "done" << endl;

  const PSInetHostAddress addr = psconfig->getRemoteServName().c_str();
  try {
    startProcServ( *psconfig, addr, psconfig->getRemotePort(), logins, vprogs );
  } catch( TCPConnection * ) {
    cerr << "Could not connect " << psconfig->getRemoteServName() << ':' << psconfig->getRemotePort() << endl;
    cerr << "Quitting ..." << endl;
  }
  
  delete psconfig;
  delete vprogs;
  delete logins;
}

void createFromXML(bool quit)
{
  PSXMLConfigFile *xmlConfigFile=createXMLConfig(xmlFile.c_str());
  cout << "XML Dump: " << endl;
  cout << "-------------------------" << endl;
  cout << *xmlConfigFile << endl;
  cout << "-------------------------" << endl;

  psconfig=makePSConfig(xmlConfigFile);
  if(!psconfig) {      
    printXMLConfigError(PSConfig::XMLENTITY_NAME);
    if( quit ) exit( 1 );
  }

  vprogs=makeVProgsConfig(xmlConfigFile);
  if(!vprogs) {     
    printXMLConfigError(VProgs::XMLENTITY_NAME);
    if( quit ) exit( 1 );
  }

  logins=makeAuthConfig(xmlConfigFile);
  if(!logins) {
    printXMLConfigError(Logins::XMLENTITY_NAME);
    if( quit ) {
      cout << "ProcServ is not in an useable configuration. Therefore quitting now." << endl;
      exit( 1 );
    }
  }
  
  delete xmlConfigFile;  
}

void startProcServ(PSConfig &psconfig, 
		   const PSInetHostAddress &addr, tpport_t port, 
		   Logins *logins, VProgs *vprogs)
{
  serv=new ProcServ(addr, port, 
		    psconfig.getLocalHostname(), psconfig.getRemoteServPassword(), 
		    logins, vprogs);
  serv->start(); 
}

PSXMLConfigFile *createXMLConfig(const char *filename)
{
  ifstream in(filename);
  PSXMLConfigFile *xmlConfigFile=new PSXMLConfigFile();
  if( !xmlConfigFile->parse( in.rdbuf() ) ) {
    cerr << "Warning: error processing XML configfile" << endl;
  }
  return xmlConfigFile;
}

bool addSignalHandlers()
{  
  return !(
	   signal(SIGINT, &sigIntHandler) == SIG_ERR |
	   signal(SIGQUIT, &sigQuitHandler) == SIG_ERR |
	   signal(SIGTERM, &sigTermHandler) == SIG_ERR |
	   signal(SIGHUP, &sigHupHandler) == SIG_ERR
	   );
}

void parseArgs(int argc, char **argv)
{
  for(int i=1; i< argc; ++i) {
    char *arg=argv[i];

    if(!strcmp(arg, OPT_XMLFILE)) {
      if(++i < argc ) {
	xmlFile=argv[i];
      }
      else printHelp(argv[0]);
    }
    else if(!strcmp(arg, OPT_HELP)) {
      printHelp(argv[0]);
    }
    else cout << "Ignoring argument: " << arg << endl;
  }
}

PSConfig *makePSConfig(PSXMLConfigFile *c)
{
  PSConfig *ret = new PSConfig(); 
  if( !c->buildPSConfig(*ret) ) {
    delete ret;
    ret = NULL;
  }

  return ret;
}

VProgs* makeVProgsConfig(PSXMLConfigFile *c)
{
  VProgs *ret = new VProgs();
  if( !c->buildVProgs(*ret) ) {
    delete ret;
    ret = NULL;
   }

  return ret;
}

Logins* makeAuthConfig(PSXMLConfigFile *c)
{
  Logins *ret = new Logins();
  if( !c->buildLogins(*ret) ) {
    delete ret;
    ret = NULL;
  }

  return ret;
}

void printHelp(const char *progName)
{
  cout << progName << " [OPTIONS]" << endl;
  cout << ' ' << OPT_XMLFILE << "    set name of configurationfile" << endl;
  cout << ' ' << OPT_HELP    << "          print this help" << endl;
  exit(1);
}

// void printConfigCheckFailed(const string &entry, bool quit)
// {
//   cout << "Config check failed for entry: " << entry << endl;
//   if(quit) exit(1);
// }

void printXMLConfigError(const string &errorEntity)
{
  cout << "Could not load entity " << errorEntity << endl
       << "Please verify your " << xmlFile << endl;
}

// void checkConfigFile(PSConfigFile &config)
// {
//   checkConfigFile(config, true);
// }

// void checkConfigFile(PSConfigFile &config, bool quit)
// {
//   int len=config.check();
//   if(len < PSConfigFile::CHECK_CONFIG_LEN) {
//     printConfigCheckFailed(PSConfigFile::CHECK_CONFIG[len], quit);
//   }

//   string timeout=config[PSConfigFile::CFG_KILLTIMEOUT];
//   if(timeout.length()>0) {
//     unsigned long ltimeout=atol(timeout.c_str());
//     if(ltimeout>0) ClientTaskSet::setDefaultKillTimeout(ltimeout);
//     else cerr << "Omitting " << timeout << " as kill timeout value" << endl;
//   }
// }

void sigIntHandler(int signum)
{
  cout << "sigIntHandler(): called with " << signum << endl;
  if(serv) serv->quitFromServer("received sig int");
}

void sigQuitHandler(int signum)
{
  cout << "sigQuitHandler() called with " << signum << endl;
  if(serv) serv->quitFromServer("received sig quit");
}

void sigTermHandler(int signum)
{
  cout << "sigTermHandler() called with " << signum << endl;
  if(serv) serv->quitFromServer("received sig term");
}

// QUICK AND DIRTY
void sigHupHandler(int signum)
{
  cout << "sigHupHandler() called with " << signum << endl;

  VProgs *oldVprogs = vprogs;
  Logins *oldLogins = logins;
  PSConfig *oldPSConfig = psconfig;

  createFromXML();
  if( psconfig ) delete psconfig; // quick and dirty
  delete oldPSConfig;

  if( !logins ) logins = oldLogins;
  else delete oldLogins;

  if( !vprogs ) vprogs = oldVprogs;
  else delete oldVprogs;
  
  serv->rehash( logins, vprogs );
  
}
