#include <psconfig.hpp>
#include <iostream>
#include <psircconnection.hpp>

const XMLToken PSConfig::XMLENTITY_NAME=               "IRC";
const XMLToken PSConfig::XMLENTITY_REMOTE_NAME=        "Remote";
const XMLToken PSConfig::XMLENTITY_REMOTE_SERVER=      "Server";
const XMLToken PSConfig::XMLENTITY_REMOTE_PORT=        "Port";
const XMLToken PSConfig::XMLENTITY_REMOTE_PASSWORD=    "Password";
const XMLToken PSConfig::XMLENTITY_LOCAL_HOSTNAME=     "Hostname";
const XMLToken PSConfig::XMLENTITY_LOCAL_NAME=         "Local";
const XMLToken PSConfig::XMLENTITY_LOCAL_DESCRIPTION=  "ServerDescription";
const XMLToken PSConfig::XMLENTITY_MANAGER_NAME=       "Manager";
const XMLToken PSConfig::XMLENTITY_MANAGER_NICK=       "Nick";
const XMLToken PSConfig::XMLENTITY_MANAGER_IDENT=      "Ident";
const XMLToken PSConfig::XMLENTITY_MANAGER_ADDRESS=    "Address";
const XMLToken PSConfig::XMLENTITY_MANAGER_REALNAME=   "Realname";
const XMLToken PSConfig::XMLENTITY_MANAGER_MODES=      "Modes";

// todo: implement value checks on manager entity
PSConfig::PSConfig() :
  remoteServName(), remotePort(0), remoteServPassword(),
  localHostname(), localServerDescription(), managerNick(),
  managerIdent(), managerAddress(), managerRealname(), managerModes()
{
#ifdef VERBOSE_MEMORY
  cout << "PSConfig] created" << endl;
#endif
}

PSConfig::PSConfig( const XMLEntity *e )
{
#ifdef VERBOSE_MEMORY
  cout << "PSConfig] created" << endl;
#endif
  process( e );
}

PSConfig::~PSConfig()
{
#ifdef VERBOSE_MEMORY
  cout << "PSConfig] destroyed" << endl;
#endif
}

bool PSConfig::process( const XMLEntity *e ) 
{
  bool ret=false;
  if( e->getType() == XMLENTITY_NAME ) {
    processXMLSubEntities( e->getSubEntities().begin(),
			  e->getSubEntities().end() );
    ret=true;
  }
  
  return ret;
}

void PSConfig::processXMLSubEntities( XMLEntity::SubEntitiesList::const_iterator begin,
				     XMLEntity::SubEntitiesList::const_iterator end)
{
  for(XMLEntity::SubEntitiesList::const_iterator it=begin; it != end; ++it) {
    if( (*it)->getType() == XMLENTITY_REMOTE_NAME )
      processRemoteXMLEntity( *(const_cast<XMLEntity*>( (*it) )) );
    else if( (*it)->getType() == XMLENTITY_LOCAL_NAME ) 
      processLocalXMLEntity( *(const_cast<XMLEntity*>( (*it) )) );
    else if( (*it)->getType() == XMLENTITY_MANAGER_NAME ) 
      processManagerXMLEntity( *(const_cast<XMLEntity*>( (*it) )) );
#ifdef DEBUG
    else cout << "PSConfig] skipping subentity: " << (*it)->getType() << endl;
#endif
  }
}

void PSConfig::processManagerXMLEntity( XMLEntity &e )
{
  managerNick     = e[XMLENTITY_MANAGER_NICK];
  PSIRCConnection::setDefaultManagerNick( managerNick );

  managerIdent    = e[XMLENTITY_MANAGER_IDENT];
  PSIRCConnection::setDefaultManagerIdent( managerIdent );

  managerAddress  = e[XMLENTITY_MANAGER_ADDRESS];
  PSIRCConnection::setDefaultManagerAddress( managerAddress );

  managerRealname = e[XMLENTITY_MANAGER_REALNAME];
  PSIRCConnection::setDefaultManagerRealname( managerRealname );

  managerModes    = e[XMLENTITY_MANAGER_MODES];
  PSIRCConnection::setDefaultManagerModes( managerModes );

#ifdef DEBUG
  cout << "PSConfig] processing manager entity ..." << endl;
  cout << "PSConfig] managerNick     = " << managerNick << endl;
  cout << "PSConfig] managerIdent    = " << managerIdent << endl;
  cout << "PSConfig] managerAddress  = " << managerAddress << endl;
  cout << "PSConfig] managerRealname = " << managerRealname << endl;
  cout << "PSConfig] managerModes    = " << managerModes << endl;
  cout << "PSConfig] done processing manager entity" << endl;
#endif
}

void PSConfig::processLocalXMLEntity( XMLEntity &e )
{
  localHostname              = e[XMLENTITY_LOCAL_HOSTNAME];
  localServerDescription     = e[XMLENTITY_LOCAL_DESCRIPTION];
  IRCServerConnection::setDefaultServerDescription( localServerDescription );

#ifdef DEBUG
  cout << "PSConfig] processing local entity ... " << endl;
  cout << "PSConfig] localHostname          = " << localHostname << endl;
  cout << "PSConfig] localServerDescription = " << localServerDescription << endl;
  cout << "PSConfig] done processing local entity ... " << endl;
#endif
}

void PSConfig::processRemoteXMLEntity( XMLEntity &e )
{
  remoteServName     = e[XMLENTITY_REMOTE_SERVER];
  remotePort         = atoi ( e[XMLENTITY_REMOTE_PORT].c_str() );
  remoteServPassword = e[XMLENTITY_REMOTE_PASSWORD];

#ifdef DEBUG
  cout << "PSConfig] processing remote entity ... " << endl;
  cout << "PSConfig] remoteServName     = " << remoteServName << endl;
  cout << "PSConfig] remotePort         = " << remotePort << endl;
  cout << "PSConfig] remoteServPassword = " << remoteServPassword << endl;
  cout << "PSConfig] done processing remote entity ... " << endl;
#endif
}
