
/***************************************************************************
                           pstask.cpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <pstask.hpp>

PSTask::PSTask( PSIRCConnection *con, ClientTaskSet *list,
		const ProcessAlias *ps, const string &options,
		bool autostart )
  : Process( ps->getCommand(), options ), ircConn( con ),
    myname ( ps->getIRCName() ), taskSet( list ), p( ps )
{
#ifdef VERBOSE_MEMORY
  cout << "PSTask] created" << endl;
#endif
  init( autostart );
}

PSTask::~PSTask()
{
#ifdef VERBOSE_MEMORY
  cout << "PSTask] destroyed" << endl;
#endif
}

void PSTask::init( bool autostart )
{ // let's hope no rehashing is performed while execution context is also in here :D
  taskSet->insert( this );
  constructName( p->getReferences() );
  p->incrementReferences();

  ircConn->introduceUser( myname, p->getIRCIdent(),
			  p->getIRCAddress(), p->getIRCModes(),
			  p->getIRCRealname() );
  ircConn->joinChannel( myname, p->getIRCChannel() );

  if( autostart ) this->start();
}

void PSTask::onTerminate()
{
  ircConn->removeUser(myname);
  taskSet->erase( this );
  if( p ) p->decrementReferences(); // -> in case of rehashs

  delete this;
}

// warning: logarithm designed for little endian architectures
void PSTask::constructName( int pid )
{
    bool ignoreNull = true;
    int erg         = 0;
    int mypid       = pid;

    // creates from an int a string representation
    for( int i=10000; i!=10; i/=10 ) {
      erg  = mypid/i;
      mypid = mypid%i;
      if(ignoreNull && erg==0) continue;
      else ignoreNull = false;      
      myname += ( '0' + erg );
    }

    if( pid != 0 ) myname += '0' + pid;
}

void PSTask::terminate()
{
#ifdef DEBUG
  cout << "PSTask] sending terminate signal to me" << endl;
#endif
  OnSignal( SIGTERM );
}

void PSTask::kill()
{
#ifdef DEBUG
  cout << "PSTask] sending kill to me " << endl;
#endif
  OnSignal( SIGKILL );
}

void PSTask::reload()
{
#ifdef DEBUG
  cout << "PSTask] sending reload signal" << endl;
#endif
  OnSignal( SIGHUP );
}
