
/***************************************************************************
                           psxmlconfigfile.cpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <psxmlconfigfile.hpp>
#include <algorithm>

const string PSXMLConfigFile::XMLENTITY_NAME = "ProcServ"; 

// todo: implement improved deletion of xml entity elements.

PSXMLConfigFile::PSXMLConfigFile()
  : XMLContainer(),
    procservEntity(NULL)
{
#ifdef VERBOSE_MEMORY
  cout << "PSXMLConfigFile] created" << endl;
#endif
}

PSXMLConfigFile::~PSXMLConfigFile()
{
#ifdef VERBOSE_MEMORY
  cout << "PSXMLConfigFile] destroyed" << endl;
#endif
}

bool PSXMLConfigFile::parse( streambuf *resource )
{
  PSXMLStream xmlStream(resource, this);
  bool ret=xmlStream.Parse();
  procservEntity=(*this)[XMLENTITY_NAME];
  
  return ret;
}

bool PSXMLConfigFile::buildVProgs( VProgs &vp )
{
  return processRepresentation( vp );
}

bool PSXMLConfigFile::buildLogins( Logins &l )
{
  return processRepresentation( l );
}

bool PSXMLConfigFile::buildPSConfig( PSConfig &p )
{
  return processRepresentation( p );
}

bool PSXMLConfigFile::processRepresentation( XMLEntityRepresentation &r )
{
  bool ret = false;
  if( procservEntity ) {
    const XMLEntity::SubEntitiesList &l = procservEntity->getSubEntities();
    if( !l.empty() ) {
      for(XMLEntity::SubEntitiesList::const_iterator it=l.begin(); it != l.end(); ++it)
	ret = ret | r.process( *it );
    }
  }

  return ret;
}

