
/***************************************************************************
                           psxmlstream.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <psxmlstream.hpp>

PSXMLStream::PSXMLStream(streambuf *buf, XMLContainer *Entities) 
  : XMLStreamImpl(buf), lastToken(UNSET), 
    lastKey(),  lastEntity(NULL), entities(Entities)
{
#ifdef VERBOSE_MEMORY
  cout << "PSXMLStream] created" << endl;
#endif
}

PSXMLStream::~PSXMLStream()
{
#ifdef VERBOSE_MEMORY
  cout << "PSXMLStream] destroyed" << endl;
#endif
}

void PSXMLStream::onStartEntity(const unsigned char *name, unsigned int len)
{
#ifdef DEBUG_XML
  cout << "PSXMLStream] onStartEntity(): " << name << endl;
#endif

  if( lastToken==ENTITY_START ) {
    bool add=lastEntity==NULL;
    lastEntity=new XMLEntity( lastEntity, lastKey );
    if( add ) entities->insert( lastEntity );
  }
  lastKey.assign( (const char*)name, len );
  lastToken=ENTITY_START;
}

void PSXMLStream::onEntityValue(const unsigned char *val, unsigned int len)
{
#ifdef DEBUG_XML
  cout << "PSXMLStream] onEntityValue(" << len << "): ";
  cout.write(val, len) << endl;
#endif


  if(len==0) return;
  else if( lastToken != ENTITY_START ) {
#ifdef DEBUG_XML
    cout << "PSXMLStream] last token not of type entity start. therefore skipping value" << endl;
#endif
    
  }
  else if( lastEntity == NULL ) {
#ifdef DEBUG_XML
    cout << "PSXMLStream] [bug ?] lastEntity==NULL. Skipping value" << endl;
#endif
  }
  else if( *val == '<' ) { // workaround some nasty bugs
#ifdef DEBUG_XML
    cout << "PSXMLStream] skipping string beginning with '<'" << endl;
#endif    
  }
  else {
    XMLToken xmlVal((const char*)val, len);
    lastEntity->addDescription( lastKey, xmlVal );
  }
}

void PSXMLStream::onEndEntity(const unsigned char *val, unsigned int len)
{
#ifdef DEBUG_XML
  cout << "PSXMLStream] onEndEntity(): " << val << endl;
#endif

  if( lastToken==ENTITY_END ) {
    lastEntity=const_cast<XMLEntity*>( lastEntity->getRoot() );
#ifdef DEBUG_XML
    if(lastEntity) cout << "PSXMLStream] new lastEntity: " << lastEntity->getType() << endl;
#endif
  }
  lastToken=ENTITY_END;
}
