
/***************************************************************************
                           xmlcontainer.cpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <algorithm>
#include "xmlcontainer.hpp"

XMLContainer::XMLContainer()
{
#ifdef VERBOSE_MEMORY
  cout << "XMLContainer] created" << endl;
#endif
}

XMLContainer::~XMLContainer()
{
  cleanUp();
#ifdef VERBOSE_MEMORY
  cout << "XMLContainer] destroyed" << endl;
#endif
}

void XMLContainer::cleanUp()
{
  for(set<XMLEntity*>::iterator it=begin(); it != end(); ++it) delete *it;
}

ostream & operator<< (ostream &out, const XMLContainer &c)
{
  for( XMLContainer::iterator it=c.begin(); it != c.end(); ++it ) {
    out << *(*it) ;
  }
  return out;
}

XMLEntity * XMLContainer::operator[] (const string &type) const
{
  XMLEntity *ret=NULL;
  iterator it=find_if( begin(), end(), XMLContainerSetPredicate(type) );
  if( it!= end() ) ret=*it;
  
  return ret;
}

XMLContainerSetPredicate::XMLContainerSetPredicate(const string &TypeName)
  : typeName(TypeName)
{
}

bool XMLContainerSetPredicate::operator() (const XMLEntity *c)
{
  return c->getType()==typeName;
}
