
/***************************************************************************
                           xmlstreamimpl.hpp
                           -------------------
    copyright            : (C) 2001 by Martin Scholl
    email                : ms@deridiot.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <xmlstreamimpl.hpp>
#include <cstring>

XMLStreamImpl::XMLStreamImpl(streambuf *buf) 
  : in(buf)
{
#ifdef VERBOSE_MEMORY
  cout << "XMLStreamImpl] created" << endl;
#endif
}

XMLStreamImpl::~XMLStreamImpl()
{
#ifdef VERBOSE_MEMORY
  cout << "XMLStreamImpl] destroyed" << endl;
#endif
}

int XMLStreamImpl::Read(unsigned char *buffer, int len)
{
  in.read( buffer, static_cast<streamsize>( len ) );
  return in.gcount();
}

void XMLStreamImpl::characters(const unsigned char *buffer, unsigned int len)
{
  const unsigned char *c;
//   for(c=buffer; c!=NULL && c!='\0' ; ++c, --len ) {
//     if( isalnum( (*c) ) || *c=='<' ) {
//       onEntityValue( c, len );
//       break;
//     }
//   }
  for( c = buffer; c != NULL && c != '\0' && 
	 ( isspace( (*c) ) || isblank( (*c) ) || iscntrl( (*c) ) )
	 ; ++c, --len );
  onEntityValue( c, len );
#ifdef DEBUG
  cout << "XMLStreamImpl] skipping: '";
  cout.write(c, len) << '\'' << endl;
#endif
}

void XMLStreamImpl::startDocument()
{
  onParserStart();
}

void XMLStreamImpl::endDocument()
{
  onParserEnd();
}


// todo: improve with while loop
void XMLStreamImpl::startElement(const unsigned char *name, 
				 const unsigned char **attr)
{
  onStartEntity( name, my_strlen(name) );
  if( attr ) {
    for( int i=0; ; ++i , ++attr) {
      if( !(*attr) ) break;
      else if(i%2) onEntityValue( *attr, my_strlen( *attr ) );
      else         onStartEntity( *attr, my_strlen( *attr ) );
    }
  }
//   onEndEntity( name );
}

void XMLStreamImpl::endElement(const unsigned char *name)
{
  onEndEntity( name, my_strlen(name) );
}

int XMLStreamImpl::my_strlen(const unsigned char *str)
{
  const unsigned char *c=str;
  int i=0;
  for(i=0; c && *c != '\0'; ++i, ++c);
  return i;
}
